import { localize2WithPath, localizeWithPath } from 'vscode/vscode/vs/nls';
import { IQuickInputService } from 'vscode/vscode/vs/platform/quickinput/common/quickInput';
import { CancellationTokenSource } from 'vscode/vscode/vs/base/common/cancellation';
import { DisposableStore } from 'vscode/vscode/vs/base/common/lifecycle';
import { Action2, MenuId } from 'vscode/vscode/vs/platform/actions/common/actions';
import { ILanguagePackService } from 'vscode/vscode/vs/platform/languagePacks/common/languagePacks';
import { ILocaleService } from 'vscode/vscode/vs/workbench/services/localization/common/locale';
import { IExtensionsWorkbenchService } from 'vscode/vscode/vs/workbench/contrib/extensions/common/extensions';

class ConfigureDisplayLanguageAction extends Action2 {
    static { this.ID = 'workbench.action.configureLocale'; }
    constructor() {
        super({
            id: ConfigureDisplayLanguageAction.ID,
            title: ( localize2WithPath(
                'vs/workbench/contrib/localization/common/localizationsActions',
                'configureLocale',
                "Configure Display Language"
            )),
            menu: {
                id: MenuId.CommandPalette
            },
            metadata: {
                description: ( localize2WithPath(
                    'vs/workbench/contrib/localization/common/localizationsActions',
                    'configureLocaleDescription',
                    "Changes the locale of VS Code based on installed language packs. Common languages include French, Chinese, Spanish, Japanese, German, Korean, and more."
                ))
            }
        });
    }
    async run(accessor) {
        const languagePackService = accessor.get(ILanguagePackService);
        const quickInputService = accessor.get(IQuickInputService);
        const localeService = accessor.get(ILocaleService);
        const extensionWorkbenchService = accessor.get(IExtensionsWorkbenchService);
        const installedLanguages = await languagePackService.getInstalledLanguages();
        const qp = quickInputService.createQuickPick();
        qp.matchOnDescription = true;
        qp.placeholder = ( localizeWithPath(
            'vs/workbench/contrib/localization/common/localizationsActions',
            'chooseLocale',
            "Select Display Language"
        ));
        if (installedLanguages?.length) {
            const items = [{ type: 'separator', label: ( localizeWithPath(
                'vs/workbench/contrib/localization/common/localizationsActions',
                'installed',
                "Installed"
            )) }];
            qp.items = items.concat(this.withMoreInfoButton(installedLanguages));
        }
        const disposables = ( new DisposableStore());
        const source = ( new CancellationTokenSource());
        disposables.add(qp.onDispose(() => {
            source.cancel();
            disposables.dispose();
        }));
        const installedSet = ( new Set(installedLanguages?.map(language => language.id) ?? []));
        languagePackService.getAvailableLanguages().then(availableLanguages => {
            const newLanguages = availableLanguages.filter(l => l.id && !( installedSet.has(l.id)));
            if (newLanguages.length) {
                qp.items = [
                    ...qp.items,
                    { type: 'separator', label: ( localizeWithPath(
                        'vs/workbench/contrib/localization/common/localizationsActions',
                        'available',
                        "Available"
                    )) },
                    ...this.withMoreInfoButton(newLanguages)
                ];
            }
            qp.busy = false;
        });
        disposables.add(qp.onDidAccept(async () => {
            const selectedLanguage = qp.activeItems[0];
            if (selectedLanguage) {
                qp.hide();
                await localeService.setLocale(selectedLanguage);
            }
        }));
        disposables.add(qp.onDidTriggerItemButton(async (e) => {
            qp.hide();
            if (e.item.extensionId) {
                await extensionWorkbenchService.open(e.item.extensionId);
            }
        }));
        qp.show();
        qp.busy = true;
    }
    withMoreInfoButton(items) {
        for (const item of items) {
            if (item.extensionId) {
                item.buttons = [{
                        tooltip: ( localizeWithPath(
                            'vs/workbench/contrib/localization/common/localizationsActions',
                            'moreInfo',
                            "More Info"
                        )),
                        iconClass: 'codicon-info'
                    }];
            }
        }
        return items;
    }
}
class ClearDisplayLanguageAction extends Action2 {
    static { this.ID = 'workbench.action.clearLocalePreference'; }
    static { this.LABEL = ( localize2WithPath(
        'vs/workbench/contrib/localization/common/localizationsActions',
        'clearDisplayLanguage',
        "Clear Display Language Preference"
    )); }
    constructor() {
        super({
            id: ClearDisplayLanguageAction.ID,
            title: ClearDisplayLanguageAction.LABEL,
            menu: {
                id: MenuId.CommandPalette
            }
        });
    }
    async run(accessor) {
        const localeService = accessor.get(ILocaleService);
        await localeService.clearLocalePreference();
    }
}

export { ClearDisplayLanguageAction, ConfigureDisplayLanguageAction };
